import 'package:bmi_global/utils/user.dart';
import 'package:flutter/material.dart';
import 'package:bmi_global/provider/app_info.dart';
import 'package:bmi_global/provider/auth.dart';
import 'package:bmi_global/provider/config.dart';
import 'package:provider/provider.dart';
import 'package:url_launcher/url_launcher_string.dart';

abstract class SettingsWidgit {
  static Widget selectTheme() => const _SelectTheme();
  static Widget appInfo<I extends AppInfoProvider>() => _AppInfo<I>();
  static Widget signOut<U extends AuthUser, A extends AuthProvider<U>>() =>
      _SignOut<U, A>();
}

class _SelectTheme extends StatelessWidget {
  const _SelectTheme({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final configProvider = Provider.of<ConfigProvider>(context);
    return ListTile(
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(15.0),
      ),
      title: const Text("Select Theam"),
      leading: const Icon(Icons.brightness_medium_rounded),
      trailing: DropdownButton<ThemeMode>(
        value: configProvider.themeMode,
        items: const [
          DropdownMenuItem(value: ThemeMode.system, child: Text("System")),
          DropdownMenuItem(value: ThemeMode.dark, child: Text("Dark")),
          DropdownMenuItem(value: ThemeMode.light, child: Text("Light")),
        ],
        onChanged: configProvider.onThemeModeChange,
      ),
    );
  }
}

class _AppInfo<I extends AppInfoProvider> extends StatelessWidget {
  const _AppInfo({Key? key}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    final appInfoProvider = Provider.of<I>(
      context,
      listen: false,
    );
    return ListTile(
      title: const Text("Current Version"),
      leading: GestureDetector(
        onTap: () async {
          await launchUrlString("https://bmi.panth.xyz/");
        },
        child: Icon(
          Icons.add_to_home_screen_rounded,
          color: Theme.of(context).colorScheme.tertiary,
        ),
      ),
      subtitle: Text(appInfoProvider.appVersion.toString()),
      trailing: const Icon(Icons.security_update_good_rounded),
    );
  }
}

class _SignOut<U extends AuthUser, A extends AuthProvider<U>>
    extends StatelessWidget {
  const _SignOut({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final logOut = Provider.of<A>(context, listen: false).logOut;
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: ElevatedButton.icon(
        style: ElevatedButton.styleFrom(
          primary: Colors.redAccent,
          padding: const EdgeInsets.all(8),
        ),
        onPressed: () {
          showDialog(
            context: context,
            builder: (context) {
              return AlertDialog(
                title: const Text("Log Out"),
                content: const Text(
                  "Are you sure you want to log out. If logged out successfully, some local info might be deleted",
                ),
                actions: [
                  TextButton(
                    onPressed: () {
                      Navigator.pop(context);
                    },
                    child: const Text("No"),
                  ),
                  TextButton(
                    onPressed: () {
                      logOut();
                      Navigator.pop(context);
                    },
                    child: const Text("Yes"),
                  ),
                ],
              );
            },
          );
        },
        icon: const Icon(Icons.logout),
        label: const Text("Sign Out"),
      ),
    );
  }
}
