import 'dart:io';
import 'package:dio/dio.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:bmi_global/provider/firebase_doc.dart';
import 'package:bmi_global/utils/parsers.dart';
import 'package:bmi_global/utils/version.dart';
import 'package:open_file/open_file.dart';

AppInfoDoc Function(RawMap) _parse(String node) {
  return (data) {
    return AppInfoDoc.fromJson(data[node].toMap());
  };
}

abstract class AppInfoProvider extends FirebaseDocProvider<AppInfoDoc> {
  var downloading = false;
  var progress = .0;
  var apkAvalable = false;
  final String _firebaseStorageApkPath;

  AppInfoProvider(this._firebaseStorageApkPath)
      : super(parse: _parse(_firebaseStorageApkPath));

  @override
  String? get path => "CONFIG/ANDROID_APK_LOG";

  Version get appVersion;
  Directory get apkDir;
  FirebaseStorage get storageInstance;

  Version get currentVersion => doc?.version ?? appVersion;

  String get _filePath => "${apkDir.path}/$currentVersion.apk";

  @override
  AppInfoDoc? convertor(Map<String, Object?>? data) {
    final newDoc = super.convertor(data);
    if (newDoc?.version != doc?.version) _checkApk();
    return newDoc;
  }

  bool get isNew {
    return currentVersion > appVersion;
  }

  void removeApks() {
    apkDir.delete(recursive: true);
  }

  void download() async {
    try {
      final dio = Dio();
      await dio.download(
        await storageInstance.ref(_firebaseStorageApkPath).getDownloadURL(),
        _filePath,
        onReceiveProgress: (rec, total) {
          downloading = true;
          progress = rec / total;
          notifyListeners();
        },
      );
      openApk();
    } catch (_) {}
    downloading = false;
    _checkApk();
    notifyListeners();
  }

  void _checkApk() {
    File(_filePath).exists().then((exists) {
      if (exists) apkAvalable = true;
      notifyListeners();
    });
  }

  void openApk() {
    OpenFile.open(_filePath);
  }
}
