import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:flutter_auth_ui/flutter_auth_ui.dart';
import 'package:bmi_global/utils/user.dart';

abstract class AuthProvider<U extends AuthUser> with ChangeNotifier {
  var _loading = true;
  FirebaseAuth get authInstance;
  U? _user;
  late final void Function() _cancle;
  final BuildContext context;
  final tosAndPrivacyPolicy = const TosAndPrivacyPolicy(
    tosUrl: "https://www.google.com",
    privacyPolicyUrl: "https://www.google.com",
  );
  final authProviders = [AuthUiProvider.phone];

  bool get loading => _loading;
  U? get user => _user;

  Future<U> userBuilder(User user);

  AuthProvider.init(this.context) {
    _cancle = authInstance.userChanges().listen((user) async {
      if (user != null) {
        try {
          _user = await userBuilder(user);
        } catch (_) {
          _user = null;
        }
      }
      _loading = false;
      notifyListeners();
    }).cancel;
  }

  void refresh() {
    authInstance.currentUser?.reload();
  }

  void logOut() {
    if (_user == null || _loading) return;
    _user = null;
    authInstance.signOut();
    notifyListeners();
  }

  Future<void> login() async {
    if (_loading) return;
    _loading = true;
    if (!await FlutterAuthUi.startUi(
      items: authProviders,
      tosAndPrivacyPolicy: tosAndPrivacyPolicy,
    )) {
      _loading = false;
      notifyListeners();
    }
  }

  @override
  void dispose() {
    _cancle();
    super.dispose();
  }
}
