import 'dart:convert';

import 'package:bmi_global/utils/advanced.dart';
import 'package:flutter/material.dart';

int toInt(String string) {
  return int.tryParse(string) ?? 0;
}

class RawObj {
  final Object? data;

  RawObj(this.data);

  static RawObj? nullable(Object? data) {
    if (data == null) return null;
    return RawObj(data);
  }

  static Map<String, RawObj?> toNullableRawMap(Map<String, Object?> data) {
    return data.map((k, v) => MapEntry(k, RawObj.nullable(v)));
  }

  IntMoney toIntMoney() {
    return IntMoney(toInt());
  }

  IntQuntity toIntQuntity() {
    return IntQuntity(toInt());
  }

  DateTimeString toDateTimeString() {
    return DateTimeString(toString());
  }

  bool toBool() {
    var obj = data;
    if (obj == null) return false;
    if (obj is bool) return obj;
    if (obj is int || obj is double) return obj != 0;
    if (obj is String) return obj.isNotEmpty;
    if (obj is Iterable) return obj.isNotEmpty;
    if (obj is Map) return obj.isNotEmpty;
    return false;
  }

  int toInt() {
    var obj = data;
    if (obj is int) return obj;
    if (obj is String) return int.tryParse(obj) ?? 0;
    if (toBool()) return 1;
    return 0;
  }

  @override
  String toString() {
    return data.toString();
  }

  RawObj? get nullOr => data == null ? null : this;

  RawMap toMap() {
    var obj = data;
    if (obj is String) {
      try {
        obj = jsonDecode(obj);
      } catch (_) {}
    }
    if (obj is List<Object?>) {
      var i = 0;
      obj = Map.fromEntries(obj.map((e) => MapEntry((i++).toString(), e)));
    }
    if (obj is Map<String, Object?>) return RawMap(obj);
    try {
      return RawMap(Map<String, Object?>.from(obj as dynamic));
    } catch (_) {
      return const RawMap({});
    }
  }

  Iterable<RawObj> toItrable() {
    var obj = data;
    if (obj is String) {
      try {
        obj = jsonDecode(obj);
      } catch (_) {}
    }
    if (obj is List) return obj.map((x) => RawObj(x));
    return [];
  }
}

@immutable
class RawMap implements Map {
  final Map<String, Object?> data;
  const RawMap(this.data);

  @override
  RawObj operator [](Object? key) {
    return RawObj(data[key]);
  }

  @override
  void operator []=(key, value) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  void addAll(Map other) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  void addEntries(Iterable<MapEntry> newEntries) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  Map<RK, RV> cast<RK, RV>() {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  void clear() {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  bool containsKey(Object? key) {
    return data.containsKey(key);
  }

  @override
  bool containsValue(Object? value) {
    return data.containsValue(value);
  }

  @override
  Iterable<MapEntry<String, RawObj>> get entries =>
      data.entries.map((e) => MapEntry(e.key, RawObj(e.value)));

  @override
  void forEach(void Function(String key, RawObj value) action) {
    for (var e in entries) {
      action(e.key, e.value);
    }
  }

  @override
  bool get isEmpty => data.isEmpty;

  @override
  bool get isNotEmpty => data.isEmpty;

  @override
  Iterable<String> get keys => data.keys;

  @override
  int get length => data.length;

  @override
  Map<K2, V2> map<K2, V2>(
      MapEntry<K2, V2> Function(String key, RawObj value) convert) {
    return data.map((key, value) => convert(key, RawObj(value)));
  }

  @override
  putIfAbsent(key, Function() ifAbsent) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  remove(Object? key) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  void removeWhere(bool Function(dynamic key, dynamic value) test) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  update(key, Function(dynamic value) update, {Function()? ifAbsent}) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  void updateAll(Function(dynamic key, dynamic value) update) {
    throw Error.throwWithStackTrace(
      "this is a read only Map",
      StackTrace.current,
    );
  }

  @override
  Iterable<RawObj> get values => data.values.map((e) => RawObj(e));
}
