import 'package:flutter/material.dart';

class HeaderTile extends StatelessWidget {
  const HeaderTile({Key? key, required this.title, this.trailing})
      : super(key: key);
  final String title;
  final Widget? trailing;

  @override
  Widget build(BuildContext context) {
    return ListTile(
      title: Text(
        title,
        style: Theme.of(context).textTheme.headline5?.merge(
              TextStyle(
                color: Theme.of(context).colorScheme.tertiary,
              ),
            ),
      ),
      trailing: trailing,
    );
  }
}

class ErrorTile extends StatelessWidget {
  const ErrorTile(this.error, {Key? key}) : super(key: key);
  final String? error;

  @override
  Widget build(BuildContext context) {
    if (error == null) return const SizedBox();
    return ListTile(
      tileColor: Colors.redAccent,
      title: Text(error ?? "Something went wrong"),
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(15.0),
      ),
    );
  }
}

class CardTile extends StatelessWidget {
  const CardTile({
    Key? key,
    required this.title,
    this.trailing = "",
    this.subtitle,
    this.leading,
    this.onTap,
    this.disabled,
  }) : super(key: key);
  final String title;
  final String? subtitle;
  final String trailing;
  final IconData? leading;
  final void Function()? onTap;
  final bool? disabled;

  @override
  Widget build(BuildContext context) {
    final theme = Theme.of(context);
    final isDark = theme.brightness == Brightness.dark;
    final textStyle = isDark
        ? const TextStyle(color: Colors.black)
        : const TextStyle(color: Colors.white);
    final subTextStyle = isDark
        ? const TextStyle(color: Colors.black87)
        : const TextStyle(color: Colors.white70);
    return Padding(
      padding: const EdgeInsets.only(bottom: 8.0),
      child: ListTile(
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(15.0),
        ),
        onTap: disabled == true ? null : onTap,
        leading: leading == null
            ? null
            : Icon(leading, color: isDark ? Colors.black : Colors.white),
        tileColor: theme.colorScheme.inverseSurface
            .withOpacity(disabled == true ? .7 : 1),
        title: Text(title, style: textStyle),
        subtitle:
            subtitle != null ? Text(subtitle ?? "", style: subTextStyle) : null,
        trailing: Text(trailing, style: textStyle),
      ),
    );
  }
}

class PopupTile<T> extends PopupMenuItem<T> {
  PopupTile({
    Key? key,
    required String child,
    required T value,
    required Widget icon,
    bool enabled = true,
    TextStyle? style,
  }) : super(
          key: key,
          child: ListTile(
            enabled: enabled,
            leading: icon,
            title: Text(child, style: style),
          ),
          enabled: enabled,
          value: value,
        );
}
